@@ -769,9 +769,12 @@ arithmetic_operator_run(struct tableparams *p,
                         struct gal_arithmetic_set_params *setprm,
                         gal_data_t **stack)
 {
+  int flags=GAL_ARITHMETIC_FLAGS_BASIC;
   gal_data_t *d1=NULL, *d2=NULL, *d3=NULL;
-  int flags = ( GAL_ARITHMETIC_INPLACE | GAL_ARITHMETIC_FREE
-                | GAL_ARITHMETIC_NUMOK );
+
+  /* Set the operating-mode flags if necessary. */
+  if(p->cp.quiet) flags |= GAL_ARITHMETIC_FLAG_QUIET;
+  if(p->envseed)  flags |= GAL_ARITHMETIC_FLAG_ENVSEED;
 
   /* When 'num_operands!=0', the operator is in the library. */
   if(token->num_operands)
@@ -926,8 +929,10 @@ arithmetic_reverse_polish(struct tableparams *p, struct column_pack *outpack)
               "single value, but other columns have also been requested "
               "which have more elements/rows");
 
-      /* Set 'single->next' to NULL so it isn't treated as a list and
-         remove all metadata */
+      /* Set 'single->next' to NULL so it isn't treated as a list, and set
+         all flags to zero (the arithmetic operation may have changed what
+         the flags pointed to). */
+      single->flag=0;
       single->next=NULL;
       gal_list_data_add(&p->table, single);
     }
