@@ -33,6 +33,7 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 
 #include <gsl/gsl_linalg.h>
 #include <wcslib/wcsmath.h>
+#include <wcslib/wcsprintf.h>
 
 #include <gnuastro/wcs.h>
 #include <gnuastro/tile.h>
@@ -123,6 +124,24 @@ wcs_read_correct_pc_cd(struct wcsprm *wcs)
 
 
 
+/* For the TPV, TNX and ZPX distortions, WCSLIB can't deal with the CDELT
+   keys properly and its better to use the CD matrix instead, this function
+   will check for this and return 1 if a CD matrix should be used
+   (over-riding the user's desired matrix if necessary). */
+static int
+wcs_use_cd_for_distortion(struct wcsprm *wcs)
+{
+  return ( wcs
+           && wcs->lin.disseq
+           && ( !strcmp(   wcs->lin.disseq->dtype[1], "TPV")
+                || !strcmp(wcs->lin.disseq->dtype[1], "TNX")
+                || !strcmp(wcs->lin.disseq->dtype[1], "ZPX") ) );
+}
+
+
+
+
+
 /* Read the WCS information from the header. Unfortunately, WCS lib is
    not thread safe, so it needs a mutex. In case you are not using
    multiple threads, just pass a NULL pointer as the mutex.
@@ -145,9 +164,9 @@ gal_wcs_read_fitsptr(fitsfile *fptr, int linearmatrix, size_t hstartwcs,
                      size_t hendwcs, int *nwcs)
 {
   /* Declaratins: */
-  int sumcheck;
   size_t i, fulllen;
   int nkeys=0, status=0;
+  int sumcheck, nocomments;
   struct wcsprm *wcs=NULL;
   char *fullheader, *to, *from;
   int fixstatus[NWCSFIX]={0};/* For the various wcsfix checks.          */
@@ -158,8 +177,16 @@ gal_wcs_read_fitsptr(fitsfile *fptr, int linearmatrix, size_t hstartwcs,
   void *fixnaxis = NULL;     /* For now disable cylfix() with this      */
                              /* (because it depends on image size).     */
 
+  /* In case the user has asked to limit the HDU keyword cards to use for
+     WCS reading, also count comment/history/empty lines (so the lines here
+     correspond to the output of 'astfits image.fits -h1'). But if no
+     limitation is requested, avoid those lines to make the processing
+     easier for WCSLIB. */
+  nocomments = hendwcs>hstartwcs ? 0 : 1;
+
   /* CFITSIO function: */
-  if( fits_hdr2str(fptr, 1, NULL, 0, &fullheader, &nkeys, &status) )
+  if( fits_hdr2str(fptr, nocomments, NULL, 0, &fullheader,
+                   &nkeys, &status) )
     gal_fits_io_error(status, NULL);
 
   /* Only consider the header keywords in the current range: */
@@ -167,11 +194,10 @@ gal_wcs_read_fitsptr(fitsfile *fptr, int linearmatrix, size_t hstartwcs,
     {
       /* Mark the last character in the desired region. */
       fullheader[hendwcs*(FLEN_CARD-1)]='\0';
-      /*******************************************************/
-      /******************************************************
+
+      /* For a check:
       printf("%s\n", fullheader);
-      ******************************************************/
-      /*******************************************************/
+      */
 
       /* Shift all the characters to the start of the string. */
       if(hstartwcs)                /* hstartwcs!=0 */
@@ -183,20 +209,19 @@ gal_wcs_read_fitsptr(fitsfile *fptr, int linearmatrix, size_t hstartwcs,
 
       nkeys=hendwcs-hstartwcs;
 
-      /*******************************************************/
-      /******************************************************
+      /* For a check:
       printf("\n\n\n###############\n\n\n\n\n\n");
       printf("%s\n", &fullheader[1*(FLEN_CARD-1)]);
       exit(0);
-      ******************************************************/
-      /*******************************************************/
+      */
     }
 
   /* WCSlib function to parse the FITS headers. */
   status=wcspih(fullheader, nkeys, relax, ctrl, &nreject, nwcs, &wcs);
   if(status)
     {
-      fprintf(stderr, "\n##################\n"
+      fprintf(stderr,"\n"
+              "##################\n"
               "WCSLIB Warning: wcspih ERROR %d: %s.\n"
               "##################\n",
               status, wcs_errmsg[status]);
@@ -320,6 +345,15 @@ gal_wcs_read_fitsptr(fitsfile *fptr, int linearmatrix, size_t hstartwcs,
                       __func__, fixstatus[CYLFIX]);
             }
 
+          /* Enable wcserr if necessary (can help in debugging situations
+             when the default error message isn't enough).  If you confront
+             an error, un-comment the two lines below and put
+             'wcsperr(wcs,NULL);' after the problematic WCSLIB
+             command. This will print a trace-back of the cause.
+          wcserr_enable(1);
+          wcsprintf_set(stderr);
+          */
+
           /* Set the WCS structure. */
           status=wcsset(wcs);
           if(status)
@@ -353,10 +387,12 @@ gal_wcs_read_fitsptr(fitsfile *fptr, int linearmatrix, size_t hstartwcs,
         }
     }
 
-  /* If the user wants a CD linear matrix, do the conversion here,
-     otherwise, make sure the PC matrix is used. */
-  if(linearmatrix==GAL_WCS_LINEAR_MATRIX_CD) gal_wcs_to_cd(wcs);
-  else                          gal_wcs_decompose_pc_cdelt(wcs);
+  /* If the distortion requires a CD matrix, or if the user wants it, do
+     the conversion here, otherwise, make sure the PC matrix is used. */
+  if(wcs_use_cd_for_distortion(wcs) \
+     || linearmatrix==GAL_WCS_LINEAR_MATRIX_CD)
+    gal_wcs_to_cd(wcs);
+  else gal_wcs_decompose_pc_cdelt(wcs);
 
   /* Clean up and return. */
   status=0;
@@ -379,7 +415,7 @@ gal_wcs_read(char *filename, char *hdu, int linearmatrix,
   struct wcsprm *wcs;
 
   /* Make sure we are dealing with a FITS file. */
-  if( gal_fits_name_is_fits(filename) == 0 )
+  if( gal_fits_file_recognized(filename) == 0 )
     return NULL;
 
   /* Check HDU for realistic conditions: */
@@ -496,70 +532,53 @@ gal_wcs_dimension_name(struct wcsprm *wcs, size_t dimension)
 /*************************************************************
  ***********               Write WCS               ***********
  *************************************************************/
-void
-gal_wcs_write_in_fitsptr(fitsfile *fptr, struct wcsprm *wcs)
+char *
+gal_wcs_write_wcsstr(struct wcsprm *wcs, int *nkeyrec)
 {
   char *wcsstr;
-  int cdfordist, status=0, nkeyrec;
-
-  /* For the TPV, TNX and ZPX distortions, WCSLIB can't deal with the CDELT
-     keys properly and its better to use the CD matrix instead, so we'll
-     use the 'gal_wcs_to_cd' function. */
-  cdfordist = ( wcs->lin.disseq
-                && ( !strcmp(   wcs->lin.disseq->dtype[1], "TPV")
-                     || !strcmp(wcs->lin.disseq->dtype[1], "TNX")
-                     || !strcmp(wcs->lin.disseq->dtype[1], "ZPX") ) );
+  int status=0;
 
   /* Finalize the linear transformation matrix. Note that some programs may
      have worked on the WCS. So even if 'altlin' is already 2, we'll just
      ensure that the final matrix is CD here. */
-  if(wcs->altlin==2 || cdfordist) gal_wcs_to_cd(wcs);
-  else                            gal_wcs_decompose_pc_cdelt(wcs);
+  if(wcs->altlin==2 || wcs_use_cd_for_distortion(wcs)) gal_wcs_to_cd(wcs);
+  else gal_wcs_decompose_pc_cdelt(wcs);
 
   /* Clean up small errors in the PC matrix and CDELT values. */
   gal_wcs_clean_errors(wcs);
 
-  /* Convert the WCS information to text. */
-  status=wcshdo(WCSHDO_safe, wcs, &nkeyrec, &wcsstr);
+  /* Convert the WCS information to text. If there was an error, then free
+     any allocated space and put zero on 'nkeyrec'. */
+  status=wcshdo(WCSHDO_safe, wcs, nkeyrec, &wcsstr);
   if(status)
-    error(0, 0, "%s: WARNING: WCSLIB error, no WCS in output.\n"
-          "wcshdu ERROR %d: %s", __func__, status, wcs_errmsg[status]);
-  else
     {
-      gal_fits_key_write_wcsstr(fptr, wcs, wcsstr, nkeyrec);
-      free(wcsstr);
+      error(0, 0, "%s: WARNING: WCSLIB error, no WCS in output.\n"
+            "wcshdo ERROR %d: %s", __func__, status, wcs_errmsg[status]);
+      if(wcsstr) free(wcsstr);
+      *nkeyrec=0;
+      wcsstr=NULL;
     }
-  status=0;
 
-   /* WCSLIB is going to write PC+CDELT keywords in any case. But when we
-      have a TPV, TNX or ZPX distortion, it is cleaner to use a CD matrix
-      (WCSLIB can't convert coordinates properly if the PC matrix is used
-      with the TPV distortion). So to help users avoid the potential
-      problems with WCSLIB. 'gal_wcs_to_cd' function already made sure that
-      CDELTi=1.0, so effectively the CD matrix and PC matrix are
-      equivalent, we just need to convert the keyword names and delete the
-      CDELT keywords. Note that zero-valued PC/CD elements may not be
-      present, so we'll manually set 'status' to zero to avoid CFITSIO from
-      crashing. */
-  if(wcs->altlin==2)
-    {
-      status=0; fits_delete_str(fptr, "CDELT1", &status);
-      status=0; fits_delete_str(fptr, "CDELT2", &status);
-      status=0; fits_modify_name(fptr, "PC1_1", "CD1_1", &status);
-      status=0; fits_modify_name(fptr, "PC1_2", "CD1_2", &status);
-      status=0; fits_modify_name(fptr, "PC2_1", "CD2_1", &status);
-      status=0; fits_modify_name(fptr, "PC2_2", "CD2_2", &status);
-      if(wcs->naxis==3)
-        {
-          status=0; fits_delete_str(fptr, "CDELT3", &status);
-          status=0; fits_modify_name(fptr, "PC1_3", "CD1_3", &status);
-          status=0; fits_modify_name(fptr, "PC2_3", "CD2_3", &status);
-          status=0; fits_modify_name(fptr, "PC3_1", "CD3_1", &status);
-          status=0; fits_modify_name(fptr, "PC3_2", "CD3_2", &status);
-          status=0; fits_modify_name(fptr, "PC3_3", "CD3_3", &status);
-        }
-      status=0;
-    }
+  /* Return the string. */
+  return wcsstr;
+}
+
+
+
+
+
+void
+gal_wcs_write_in_fitsptr(fitsfile *fptr, struct wcsprm *wcs)
+{
+  char *wcsstr;
+  int nkeyrec=0;
+
+  /* Convert the WCS structure into FITS keywords as a string. */
+  wcsstr=gal_wcs_write_wcsstr(wcs, &nkeyrec);
+
+  /* Write the keywords into the FITS file. */
+  gal_fits_key_write_wcsstr(fptr, wcs, wcsstr, nkeyrec);
+  free(wcsstr);
 }
 
 
@@ -579,7 +598,7 @@ gal_wcs_write(struct wcsprm *wcs, char *filename,
   /* Small sanity checks */
   if(wcs==NULL)
     error(EXIT_FAILURE, 0, "%s: input WCS is NULL", __func__);
-  if( gal_fits_name_is_fits(filename)==0 )
+  if( gal_fits_file_recognized(filename)==0 )
     error(EXIT_FAILURE, 0, "%s: not a FITS suffix", filename);
 
   /* Open the file for writing */
@@ -959,9 +978,8 @@ gal_wcs_coordsys_convert(struct wcsprm *wcs, int outcoordsys)
   char *alt=NULL;                 /* Only concerned with primary wcs. */
   double equinox=0.0f;            /* To preserve current value.       */
   struct wcsprm *out=NULL;
-  char *clng, *clat, *radesys;
   double lng2p1=NAN, lat2p1=NAN, lng1p2=NAN;
-
+  char *clng=NULL, *clat=NULL, *radesys=NULL;
 
   /* Just incase the input is a NULL pointer. */
   if(wcs==NULL) return NULL;
@@ -1388,7 +1406,6 @@ gal_wcs_remove_dimension(struct wcsprm *wcs, size_t fitsdim)
 #endif
   /**************************************************/
 
-
   /* First loop over the arrays. */
   for(i=0;i<naxis;++i)
     {
